﻿package com.robotacid.engine {
	
	/**
	* Particle
	* @author Aaron Steed
	* 
	* Free floating Verlet Integration particle
	*/
	public class Particle extends Dot{
		public var px:Number;//		previous x position
		public var py:Number;//		previous y position
		public var temp_x:Number;//		temp variable
		public var temp_y:Number;//		temp variable
		// Constructor
		public function Particle(x:Number, y:Number){
			super(x, y);
			px = temp_x = x;
			py = temp_y = y;
		}
		// Move the particle
		public function verlet(gravity_x:Number, gravity_y:Number, damping:Number):void{
			temp_x = x;
			temp_y = y;
			x += damping * (x - px) + gravity_x;
			y += damping * (y - py) + gravity_y;
			px = temp_x;
			py = temp_y;
		}
		// Set the position
		public function setPosition(x:Number, y:Number):void{
			this.x = px = temp_x = ix = x;
			this.y = py = temp_y = iy = y;
		}
		// Fix the particle to it's start position
		override public function pin():void{
			x = px = temp_x = ix;
			y = py = temp_y = iy;
		}
		// Fix the particle to a given position
		public function pinTo(x:Number, y:Number):void{
			this.x = ix = px = temp_x = x;
			this.y = iy = py = temp_y = y;
		}
		// Create a Line describing the last frame of movement
		public function getLine():Line{
			return new Line(new Dot(px, py), this);
		}
		// Calculate current y speed
		public function yspeed():Number{
			return y-py;
		}
		// Calculate current x speed
		public function xspeed():Number{
			return x-px;
		}
		// Add velocity to the particle
		public function addVelocity(x:Number, y:Number):void{
			px -= x;
			py -= y;
		}
		// Generate a copy of this particle with duplicate velocity registered to the same physics engine
		public function copy():Particle{
			var temp = new Particle(x, y);
			temp.px = px;
			temp.py = py;
			return temp;
		}
		// Add counter-thrust to the direction it last travelled
		public function reverse(strength:Number):void{
			var vx:Number = px-x;
			var vy:Number = py-y;
			addVelocity(vx * strength, vy * strength);
		}
		// Return a string describing the current and previous location of this Particle
		public function toString():String{
			return "("+x+","+y+") "+x+","+y;
		}
		
	}
	
}