﻿package com.robotacid.engine_specific {
	import com.robotacid.engine.Particle;
	import com.robotacid.engine.Trig;
	import com.robotacid.engine.Util;
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.display.Shape;
	
	/**
	* ...
	* @author Aaron Steed
	*/
	public class Missile extends Particle{
		
		public var mc:Shape;
		public var gfx:Graphics;
		public var g:Game;
		public var program:Array;
		public var subroutines:Array;
		public var index:int;
		public var sub_index:int;
		public var start_index:int;
		public var count:int;
		public var delay:int;
		public var r:int;
		public var thrust:Number;
		public var radius:Number;
		public var over:Boolean;
		public var active:Boolean;
		public var health:int;
		public var parent:Missile;
		public var evasive:Boolean;
		public var friendly:Boolean;
		public var spawning:Boolean;
		public var fitness:Number;
		public var speed:Number;
		public var age:int;
		public var strings:Array;
		
		private var i:int;
		
		public static const GRAVITY_X:Number = 0.0;
		public static const GRAVITY_Y:Number = 0.0;
		public static const DAMPING:Number = 0.99;
		public static const HEALTH:int = 3;
		public static const RADIUS:int = 10;
		public static const MAX_SPEED:Number = 3;
		public static const FRIENDLY_AGE:int = 5;
		public static const NON_SPAWNING_AGE:int = 10;
		public static const NON_EVASIVE_AGE:int = 20;
		public static const SPEED_STEP:Number = 1 / 10;
		
		public function Missile(health:int, mc:Shape, program:Array, subroutines:Array, strings:Array, g:Game, parent:Missile = null) {
			super(mc.x, mc.y);
			this.mc = mc;
			gfx = mc.graphics;
			this.g = g;
			this.program = program;
			this.subroutines = subroutines;
			this.strings = strings;
			this.parent = parent;
			radius = RADIUS;
			// command reading always starts after the identity commands
			this.health = health;
			evasive = false;
			friendly = true;
			spawning = false;
			age = 0;
			if(parent != null) {
				evasive = parent.evasive;
				friendly = parent.friendly;
				spawning = parent.spawning;
				age = parent.age;
			}
			reset();
			if(program.length > 0) readProgram();
		}
		public function main():void {
			over = false;
			if(g.launcher.active && friendly) {
				if(x > g.gun.x) addVelocity( -0.1, 0);
				else if(x < g.gun.x) addVelocity(0.1, 0);
			}
			if(!g.launcher.active && y > (Math.random() * 200) + Game.HEIGHT * 0.5) {
				addVelocity(0, -Math.random() * 3);
			}
			verlet(GRAVITY_X, GRAVITY_Y, DAMPING);
			if(x > Game.WIDTH - (radius + health)) x = Game.WIDTH - (radius + health);
			if(x < radius + health) x = (radius + health);
			if(y < -radius - health) y = (-radius - health);
			count++;
			if(count >= delay && program.length > 0) readProgram();
			if(Util.proximity(g.canvas.mouseX, g.canvas.mouseY, x, y, radius+health)) {
				over = true;
			}
			updateFitness();
			updateMC();
		}
		public function reset():void {
			pin();
			health = HEALTH;
			index = start_index = 0;
			count = 0;
			thrust = 0;
			fitness = 0;
			over = false;
			active = true;
			speed = SPEED_STEP * age;
			speed = Math.min(speed, 1);
			if(friendly && age > FRIENDLY_AGE) friendly = false;
			if(!evasive && age > NON_EVASIVE_AGE) evasive = true;
			if(!spawning && age > NON_SPAWNING_AGE) spawning = true;
			draw();
			updateMC();
		}
		// Update success ratio of missiles - spawn of missiles confer success upon their parents
		public function updateFitness():void {
			if(y > fitness) fitness = y;
			if(parent != null && fitness > parent.fitness) parent.fitness = fitness;
		}
		// Read a logo-based instruction set - the W command exits the reading session
		public function readProgram():void{
			for(i = 0; i < 100; i++){
				// F: forwards
				if(program[index] == "F"){
					index++;
					thrust = 0.5 + program[index] * speed;
					addVelocity(thrust * Trig.cos[r], thrust * Trig.sin[r]);
					index++;
				// S: spawn a monster
				} else if(program[index] == "S"){
					index++;
					if(spawning){
						sub_index = program[index];
						if(health > 1){
							g.launcher.spawnMissile(health-1, this, subroutines[sub_index], subroutines);
						}
					}
					index++;
				// R: rotate to
				} else if(program[index] == "R"){
					index++;
					r = program[index];
					while(r >= 360) r -= 360;
					while(r <= -1) r += 360;
					/*g.debug.lineStyle(1, 0);
					g.debug.moveTo(Game.WIDTH * 0.5, Game.HEIGHT * 0.5);
					g.debug.lineTo((Game.WIDTH * 0.5) + Trig.cos[r] * 100, (Game.HEIGHT * 0.5) + Trig.sin[r] * 100);
					*/
					index++;
				// W: wait
				} else if(program[index] == "W"){
					index++;
					delay = program[index];
					count = 0;
					break;
				// X: erase all preceding instructions
				} else if(program[index] == "X"){
					start_index = index + 1;
					index = start_index;
				} else {
					index++;
				}
				if(index >= program.length){
					index = start_index;
				}
				if(i == 99) trace("hi");
			}
			if(index >= program.length){
				index = start_index;
			}
		}
		public function hit():void {
			health--;
			draw();
		}
		public function draw():void {
			gfx.clear();
			gfx.lineStyle(0, 0, 0);
			gfx.beginFill(Game.FOREGROUND_COL);
			gfx.drawCircle(0, 0, radius + health);
			gfx.endFill();
			for(var i:int = 0; i < age; i++) {
				gfx.beginFill(Game.BACKGROUND_COL);
				gfx.drawCircle(Trig.cos[(90 + (i * 10)) % 360] * radius * 0.5, Trig.sin[(90 + (i * 10)) % 360] * radius * 0.5, 1);
				gfx.endFill();
			}
		}
		public function updateMC():void {
			mc.x = x;
			mc.y = y;
		}
		public function save():Object {
			var temp:String = "";
			temp += age + ":";
			temp += ix + ":";
			temp += iy + ":";
			for(var i:int = 0; i < strings.length; i++) {
				temp += strings[i];
				if(i < strings.length - 1) {
					temp += ":";
				}
			}
			return temp;
			//return {strings:strings, age:age, ix:ix, iy:iy};
		}
		
	}
	
}